
import sys
import glob
import os.path
import ctypes
import ctypes.util
import threading

#https://github.com/ahupp/python-magic

from ctypes import c_char_p, c_int, c_size_t, c_void_p


class MagicException(Exception):
    def __init__(self, message):
        super(MagicException, self).__init__(message)
        self.message = message


class Magic:
    """
    Magic is a wrapper around the libmagic C library.
    """

    def __init__(self, mime=False, magic_file=None, mime_encoding=False,
                 keep_going=False, uncompress=False):
        """
        Create a new libmagic wrapper.
        mime - if True, mimetypes are returned instead of textual descriptions
        mime_encoding - if True, codec is returned
        magic_file - use a mime database other than the system default
        keep_going - don't stop at the first match, keep going
        uncompress - Try to look inside compressed files.
        """
        self.flags = MAGIC_NONE
        if mime:
            self.flags |= MAGIC_MIME
        if mime_encoding:
            self.flags |= MAGIC_MIME_ENCODING
        if keep_going:
            self.flags |= MAGIC_CONTINUE

        if uncompress:
            self.flags |= MAGIC_COMPRESS

        self.cookie = magic_open(self.flags)
        self.lock = threading.Lock()

        magic_load(self.cookie, magic_file)

    def from_buffer(self, buf):
        """
        Identify the contents of `buf`
        """
        with self.lock:
            try:
                return maybe_decode(magic_buffer(self.cookie, buf))
            except MagicException as e:
                return self._handle509Bug(e)

    def from_file(self, filename):
        # raise FileNotFoundException or IOError if the file does not exist
        with open(filename):
            pass
        with self.lock:
            try:
                return maybe_decode(magic_file(self.cookie, filename))
            except MagicException as e:
                return self._handle509Bug(e)

    def _handle509Bug(self, e):
        # libmagic 5.09 has a bug where it might fail to identify the
        # mimetype of a file and returns null from magic_file (and
        # likely _buffer), but also does not return an error message.
        if e.message is None and (self.flags & MAGIC_MIME):
            return "application/octet-stream"
        else:
            raise e

    def __del__(self):
        # no _thread_check here because there can be no other
        # references to this object at this point.

        # during shutdown magic_close may have been cleared already so
        # make sure it exists before using it.

        # the self.cookie check should be unnecessary and was an
        # incorrect fix for a threading problem, however I'm leaving
        # it in because it's harmless and I'm slightly afraid to
        # remove it.
        if self.cookie and magic_close:
            magic_close(self.cookie)
            self.cookie = None


_instances = {}


def _get_magic_type(mime):
    i = _instances.get(mime)
    if i is None:
        i = _instances[mime] = Magic(mime=mime)
    return i


def from_file(filename, mime=False):
    """
    Accepts a filename and returns the detected filetype.  Return
    value is the mimetype if mime=True, otherwise a human readable
    name.
    #>>> magic.from_file("testdata/test.pdf", mime=True)
    'application/pdf'
    """
    m = _get_magic_type(mime)
    return m.from_file(filename)


def from_buffer(buffer, mime=False):
    """
    Accepts a binary string and returns the detected filetype.  Return
    value is the mimetype if mime=True, otherwise a human readable
    name.
    #>>> magic.from_buffer(open("testdata/test.pdf").read(1024))
    'PDF document, version 1.2'
    """
    m = _get_magic_type(mime)
    return m.from_buffer(buffer)


libmagic = None
# Let's try to find magic or magic1
dll = ctypes.util.find_library('magic') or ctypes.util.find_library('magic1') or ctypes.util.find_library('cygmagic-1')

# This is necessary because find_library returns None if it doesn't find the library
if dll:
    libmagic = ctypes.CDLL(dll)

if not libmagic or not libmagic._name:
    windows_dlls = ['magic1.dll', 'cygmagic-1.dll']
    platform_to_lib = {'darwin': ['/opt/local/lib/libmagic.dylib',
                                  '/usr/local/lib/libmagic.dylib',
                                  '/usr/local/Cellar/libmagic/5.31/lib/libmagic.dylib'] +
                                 # Assumes there will only be one version installed
                                 glob.glob('/usr/local/Cellar/libmagic/*/lib/libmagic.dylib'),
                       'win32': windows_dlls,
                       'cygwin': windows_dlls,
                       'linux': ['libmagic.so.1'],
                       # fallback for some Linuxes (e.g. Alpine) where library search does not work
                       }
    platform = 'linux' if sys.platform.startswith('linux') else sys.platform
    for dll in platform_to_lib.get(platform, []):
        try:
            libmagic = ctypes.CDLL(dll)
            break
        except OSError:
            pass

if not libmagic or not libmagic._name:
    # It is better to raise an ImportError since we are importing magic module
    raise ImportError('failed to find libmagic.  Check your installation')

magic_t = ctypes.c_void_p


def errorcheck_null(result, func, args):
    if result is None:
        err = magic_error(args[0])
        raise MagicException(err)
    else:
        return result


def errorcheck_negative_one(result, func, args):
    if result is -1:
        err = magic_error(args[0])
        raise MagicException(err)
    else:
        return result


# return str on python3.  Don't want to unconditionally
# decode because that results in unicode on python2
def maybe_decode(s):
    if str == bytes:
        return s
    else:
        return s.decode('utf-8')


def coerce_filename(filename):
    if filename is None:
        return None

    # ctypes will implicitly convert unicode strings to bytes with
    # .encode('ascii').  If you use the filesystem encoding
    # then you'll get inconsistent behavior (crashes) depending on the user's
    # LANG environment variable
    is_unicode = (sys.version_info[0] <= 2 and
                  isinstance(filename, unicode)) or \
                 (sys.version_info[0] >= 3 and
                  isinstance(filename, str))
    if is_unicode:
        return filename.encode('utf-8')
    else:
        return filename


magic_open = libmagic.magic_open
magic_open.restype = magic_t
magic_open.argtypes = [c_int]

magic_close = libmagic.magic_close
magic_close.restype = None
magic_close.argtypes = [magic_t]

magic_error = libmagic.magic_error
magic_error.restype = c_char_p
magic_error.argtypes = [magic_t]

magic_errno = libmagic.magic_errno
magic_errno.restype = c_int
magic_errno.argtypes = [magic_t]

_magic_file = libmagic.magic_file
_magic_file.restype = c_char_p
_magic_file.argtypes = [magic_t, c_char_p]
_magic_file.errcheck = errorcheck_null


def magic_file(cookie, filename):
    return _magic_file(cookie, coerce_filename(filename))


_magic_buffer = libmagic.magic_buffer
_magic_buffer.restype = c_char_p
_magic_buffer.argtypes = [magic_t, c_void_p, c_size_t]
_magic_buffer.errcheck = errorcheck_null


def magic_buffer(cookie, buf):
    return _magic_buffer(cookie, buf, len(buf))


_magic_load = libmagic.magic_load
_magic_load.restype = c_int
_magic_load.argtypes = [magic_t, c_char_p]
_magic_load.errcheck = errorcheck_negative_one


def magic_load(cookie, filename):
    return _magic_load(cookie, coerce_filename(filename))


magic_setflags = libmagic.magic_setflags
magic_setflags.restype = c_int
magic_setflags.argtypes = [magic_t, c_int]

magic_check = libmagic.magic_check
magic_check.restype = c_int
magic_check.argtypes = [magic_t, c_char_p]

magic_compile = libmagic.magic_compile
magic_compile.restype = c_int
magic_compile.argtypes = [magic_t, c_char_p]

MAGIC_NONE = 0x000000  # No flags
MAGIC_DEBUG = 0x000001  # Turn on debugging
MAGIC_SYMLINK = 0x000002  # Follow symlinks
MAGIC_COMPRESS = 0x000004  # Check inside compressed files
MAGIC_DEVICES = 0x000008  # Look at the contents of devices
MAGIC_MIME = 0x000010  # Return a mime string
MAGIC_MIME_ENCODING = 0x000400  # Return the MIME encoding
MAGIC_CONTINUE = 0x000020  # Return all matches
MAGIC_CHECK = 0x000040  # Print warnings to stderr
MAGIC_PRESERVE_ATIME = 0x000080  # Restore access time on exit
MAGIC_RAW = 0x000100  # Don't translate unprintable chars
MAGIC_ERROR = 0x000200  # Handle ENOENT etc as real errors

MAGIC_NO_CHECK_COMPRESS = 0x001000  # Don't check for compressed files
MAGIC_NO_CHECK_TAR = 0x002000  # Don't check for tar files
MAGIC_NO_CHECK_SOFT = 0x004000  # Don't check magic entries
MAGIC_NO_CHECK_APPTYPE = 0x008000  # Don't check application type
MAGIC_NO_CHECK_ELF = 0x010000  # Don't check for elf details
MAGIC_NO_CHECK_ASCII = 0x020000  # Don't check for ascii files
MAGIC_NO_CHECK_TROFF = 0x040000  # Don't check ascii/troff
MAGIC_NO_CHECK_FORTRAN = 0x080000  # Don't check ascii/fortran
MAGIC_NO_CHECK_TOKENS = 0x100000  # Don't check ascii/tokens


import argparse,sys
import constrains_spidy.contrain_checker as constrains

class command_line:
      def __init__(self,baseline={}):
          self.__baseline__ = baseline

      def argument_parser(self):
          ### Check Metadata
          metadata_header = ["args_list","args_var","args_action","args_help","args_required"]
          constrain = constrains.constrain_check()
          constrain.metadata_checks(self.__baseline__,metadata_header)

          ### Metadata Value Checks
          if not type(self.__baseline__["args_list"]) == list \
             or not type(self.__baseline__["args_var"]) == list \
             or not type(self.__baseline__["args_action"]) == list \
             or not type(self.__baseline__["args_help"]) == list \
             or not type(self.__baseline__["args_required"]) == list:
             print("class :- command line Function :- argument_parser Metadata Variables missing")
             sys.exit(0)

          ### Count match of Values
          if not len(self.__baseline__["args_list"]) == len(self.__baseline__["args_var"]) \
             or not len(self.__baseline__["args_list"]) == len(self.__baseline__["args_action"]) \
             or not len(self.__baseline__["args_list"]) == len(self.__baseline__["args_help"]) \
             or not len(self.__baseline__["args_list"]) == len(self.__baseline__["args_required"]):
             print("class :- command line Function :- argument_parser Metadata Values Value count mismatch")
             sys.exit(0)

          ### Parser Formation code.
          parser = argparse.ArgumentParser()
          for count in range(len(self.__baseline__["args_list"])):
              if self.__baseline__["args_required"][count] == "True":
                 parser.add_argument(self.__baseline__["args_list"][count],self.__baseline__["args_var"][count], \
                                  action=self.__baseline__["args_action"][count], help=self.__baseline__["args_help"][count], \
                                  required=self.__baseline__["args_required"][count])
              else:
                 parser.add_argument(self.__baseline__["args_list"][count], self.__baseline__["args_var"][count], \
                                      action=self.__baseline__["args_action"][count],
                                      help=self.__baseline__["args_help"][count])

          return parser



import sys
from termcolor import colored
import inspect
import socket
import os
import FILE_spidy.magic as magic
import subprocess

class constrain_check:

      def metadata_checks(self,baseline={},parameters=[]):

          missing_list__ = []

          if not type(baseline) == dict   or not type(parameters) == list:
             print(colored("Value Error :- baseline should be a Dictionary and parameters should be a List ",attrs=['reverse']))
             self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0],inspect.stack()[2][1])
             sys.exit(1)

          for parameter in parameters:
              if not baseline.keys().__contains__(parameter):
                 missing_list__.append(parameter)

          if not missing_list__.__len__() == 0:
             print (colored("Metadata Problem Library Dependencies Missing",'red',attrs=['reverse']))
             print(colored("Missing Baseline Dictionary Variables are : " + str(missing_list__)))
             self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0],inspect.stack()[2][1])
             sys.exit(1)

      def check_network(self,address,port):
           try:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.settimeout(1)
                test_conn = sock.connect_ex((address,int(port)))
                if not test_conn == 0:
                    print(colored("Check Network Settings/Service Status ;  Connectivity issues " + address + ":" + port + " not reachable",attrs=['reverse']))
                    self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0],inspect.stack()[2][1])
                    sys.exit(1)
           except OverflowError:
                print(colored("Check Network Settings/Service Status ; Connectivity issues " + address + ":" + port + " not reachable",attrs=['reverse']))
                self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0],inspect.stack()[2][1])
                sys.exit(1)

      def check_file_exists(self,file_path):

          ### File Exists check
          if (not os.path.exists(file_path)):
             print(colored("Specified File/File Path " + file_path + " does not exist", 'magenta',attrs=['reverse']))
             self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0], inspect.stack()[2][1])
             sys.exit(1)

      def check_file_type(self,file_path,file_type):

          ### File Type Constrain check
          file_format = magic.from_file(file_path, mime=True)

          if (not file_format == file_type):
              print(colored("Cannot Parse File Content " + file_path + " contains : " + file_format,'magenta',attrs=['reverse']))
              sys.exit()

      def mongo_collection_creation_constrain(self,data):

          ### Check Data Format
          if not type(data) == list or not type(data[0]) == dict:
              print(colored("Data Format Errors for MongoDb Collection Creation Kindly check the data being inserted", 'magenta',attrs=['reverse']))
              self.print_error_info(inspect.stack()[2][1].split("/")[-1].split(".")[0], inspect.stack()[2][1])
              sys.exit(1)

      def check_command_exists(self,command=[]):

          ### commands check
          check_list = []
          for cmd in command:
              check_list.append(subprocess.call("type " + cmd, shell=True,
                                     stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0)
          count = 0
          missing_commands = []
          for results in check_list:
              if not results == True:
                 missing_commands.append(command[count])
              count = count + 1

          if not missing_commands.__len__() == 0:
             print(colored("Commands missing for safe execution of program :- ",'blue',attrs=['reverse']))
             print(missing_commands)
             sys.exit(1)

      def fix_yaml_column(self,yaml_file):
          self.check_file_exists(yaml_file)

      def print_error_info(self,class_name,file_name):
          print()
          print(colored("Calling Object/Class Name :- " + class_name))
          print(colored("Class File location : " + file_name))
          print()
